# coding: utf-8

import logging

def get_logger(name: str, level: int = logging.INFO, filename: str = "systemmonitor.log"):
    """
    获取一个日志记录器，可以用于记录系统监控信息。

    Args:
        name: 日志记录器名称
        level: 日志记录器级别，默认INFO
        filename: 日志记录文件名，默认"systemmonitor.log"
    """
    logger = logging.getLogger(name)
    logger.setLevel(level)
    console_handler = logging.StreamHandler()
    file_handler = logging.FileHandler(filename) if filename != "" else None
    console_handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
    file_handler.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s'))
    logger.addHandler(console_handler)
    logger.addHandler(file_handler)
    return logger


logger = get_logger(__name__, level=logging.INFO, filename="systemmonitor.log")


# coding: utf-8

from typing import List
from dataclasses import dataclass, field
import time

from .psutil_helper import get_cpu_percent, get_memory_used


@dataclass
class Arguments:
    host: str = "0.0.0.0"
    port: int = 8000
    max_num: int = 3000
    interval: float = 1.0

    def to_dict(self):
        return {
            "host": self.host,
            "port": self.port,
            "max_num": self.max_num,
            "interval": self.interval,
        }

@dataclass
class SystemInfo:
    cpu_percent: float
    memory_used: float
    timestamp: float
    gpu: List["GPUInfo"] = field(default_factory=list)

    def to_dict(self):
        return {
            "cpu_percent": self.cpu_percent,
            "memory_used": self.memory_used,
            "timestamp": self.timestamp,
            "gpu": [g.to_dict() for g in self.gpu],
        }

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            cpu_percent=data["cpu_percent"],
            memory_used=data["memory_used"],
            timestamp=data["timestamp"],
            gpu=[GPUInfo.from_dict(g) for g in data.get("gpu", [])],
        )
    
@dataclass
class CPUStatistic:
    min: float = 0.0    
    max: float = 0.0
    avg: float = 0.0
    def to_dict(self):
        return {
            "min": self.min,
            "max": self.max,
            "avg": self.avg,
        }
    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            min=data["min"],
            max=data["max"],
            avg=data["avg"],
        )

@dataclass
class MemoryStatistic:
    min: float = 0.0
    max: float = 0.0
    avg: float = 0.0
    def to_dict(self):
        return {
            "min": self.min,
            "max": self.max,
            "avg": self.avg,
        }
    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            min=data["min"],
            max=data["max"],
            avg=data["avg"],
        )


@dataclass
class GPUStatistic:
    min: float = 0.0
    max: float = 0.0
    avg: float = 0.0

    def to_dict(self):
        return {
            "min": self.min,
            "max": self.max,
            "avg": self.avg,
        }

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            min=data["min"],
            max=data["max"],
            avg=data["avg"],
        )


@dataclass
class GPUInfo:
    """单块 GPU 的占用率信息"""
    index: int = 0
    name: str = ""
    gpu_util: int = 0
    mem_usage: float = 0.0
    mem_util: int = 0
    mem_used_mb: float = 0.0
    mem_total_mb: float = 0.0

    def to_dict(self):
        return {
            "index": self.index,
            "name": self.name,
            "gpu_util": self.gpu_util,
            "mem_usage": self.mem_usage,
            "mem_util": self.mem_util,
            "mem_used_mb": self.mem_used_mb,
            "mem_total_mb": self.mem_total_mb,
        }

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            index=data.get("index", 0),
            name=data.get("name", ""),
            gpu_util=data.get("gpu_util", 0),
            mem_usage=data.get("mem_usage", 0.0),
            mem_util=data.get("mem_util", 0),
            mem_used_mb=data.get("mem_used_mb", 0.0),
            mem_total_mb=data.get("mem_total_mb", 0.0),
        )


@dataclass
class Statistic:
    cpu: CPUStatistic
    memory: MemoryStatistic
    data_list: List[SystemInfo]
    total_length: int
    gpu_statistic: GPUStatistic
    gpu: List[GPUInfo] = field(default_factory=list)

    def to_dict(self):
        return {
            "cpu": self.cpu.to_dict(),
            "memory": self.memory.to_dict(),
            "data_list": [system_info.to_dict() for system_info in self.data_list],
            "total_length": self.total_length,
            "gpu_statistic": self.gpu_statistic.to_dict(),
            "gpu": [g.to_dict() for g in self.gpu],
        }

    @classmethod
    def from_dict(cls, data: dict):
        return cls(
            cpu=CPUStatistic.from_dict(data["cpu"]),
            memory=MemoryStatistic.from_dict(data["memory"]),
            data_list=[SystemInfo.from_dict(system_info) for system_info in data["data_list"]],
            total_length=data["total_length"],
            gpu_statistic=GPUStatistic.from_dict(
                data.get("gpu_statistic", {"min": 0.0, "max": 0.0, "avg": 0.0})
            ),
            gpu=[GPUInfo.from_dict(g) for g in data.get("gpu", [])],
        )

class SystemInfoCollector:
    def __init__(self, max_system_info_list_length: int = 3000):
        self.system_info_list: List[SystemInfo] = []
        self.max_system_info_list_length = max_system_info_list_length

    @property
    def total_length(self) -> int:
        return len(self.system_info_list)

    @property
    def latest_system_info(self) -> SystemInfo:
        return self.system_info_list[-1] if self.system_info_list else None

    def collect(self) -> SystemInfo:
        # 延迟导入，避免 schema 与 gpu_helper 的循环导入问题
        from .gpu_helper import get_gpu_info

        system_info = SystemInfo(
            get_cpu_percent(), get_memory_used(), time.time(), get_gpu_info())
        self.system_info_list.append(system_info)
        if len(self.system_info_list) > self.max_system_info_list_length:
            self.system_info_list.pop(0)
        return system_info

    def get_system_info_list(self) -> List[SystemInfo]:
        return self.system_info_list

    def reset_system_info_list(self) -> None:
        self.system_info_list.clear()

    def statistic(self) -> dict:
        # 计算GPU显存占用，合并多张显卡的显存用量
        gpu_mem_used_mb_list = [
            sum(g.mem_used_mb for g in system_info.gpu)
            for system_info in self.system_info_list
            if system_info.gpu
        ]
        if self.total_length == 0:
            statistic = Statistic(
                cpu=CPUStatistic(),
                memory=MemoryStatistic(),
                data_list=[],
                total_length=0,
                gpu_statistic=GPUStatistic(),
                gpu=[],
            )
        else:
            statistic = Statistic(
                cpu=CPUStatistic(
                    min=min(system_info.cpu_percent for system_info in self.system_info_list),
                    max=max(system_info.cpu_percent for system_info in self.system_info_list),
                    avg=sum(system_info.cpu_percent for system_info in self.system_info_list) / len(self.system_info_list),
                ),
                memory=MemoryStatistic(
                    min=min(system_info.memory_used for system_info in self.system_info_list),
                    max=max(system_info.memory_used for system_info in self.system_info_list),
                    avg=sum(system_info.memory_used for system_info in self.system_info_list) / len(self.system_info_list),
                ),
                gpu=GPUStatistic(
                    min=min(gpu_mem_used_mb_list) if gpu_mem_used_mb_list else 0.0,
                    max=max(gpu_mem_used_mb_list) if gpu_mem_used_mb_list else 0.0,
                    avg=sum(gpu_mem_used_mb_list) / len(gpu_mem_used_mb_list) if gpu_mem_used_mb_list else 0.0,
                ),
                data_list=self.system_info_list,
                total_length=self.total_length,
            )
        return statistic.to_dict()


import pytest
import requests
import json
from functools import partial

json_dump = partial(json.dumps, indent=4, ensure_ascii=False)


def test_get_system_info():
    response = requests.get("http://localhost:8000/")
    print(response.json())
    assert response.status_code == 200
    assert "cpu_percent" in response.json()
    assert "memory_percent" in response.json()

def test_start_monitoring():
    response = requests.get("http://localhost:8000/start-monitoring")
    print(response.json())
    assert response.status_code == 200
    assert "message" in response.json()
    assert response.json()["message"] == "监控已启动"

def test_stop_monitoring():
    response = requests.get("http://localhost:8000/stop-monitoring")
    print(response.json())
    assert response.status_code == 200
    assert "message" in response.json()
    assert response.json()["message"] == "监控已停止"

def test_statistic():
    response = requests.get("http://localhost:8000/statistic")
    print(json_dump(response.json()))
    assert response.status_code == 200
    assert "cpu_percent" in response.json()
    assert "memory_percent" in response.json()
    assert "data_list" in response.json()




# coding: utf-8
"""GPU 信息采集模块，基于 pynvml 获取 NVIDIA GPU 占用率等"""

from typing import List

from .schema import GPUInfo

_pynvml_available = False
try:
    import pynvml
    _pynvml_available = True
except ImportError:
    pass


def get_gpu_info() -> List[GPUInfo]:
    """
    获取所有 NVIDIA GPU 的占用率等信息。

    Returns:
        GPUInfo 列表，若无 GPU 或 pynvml 不可用则返回空列表。
    """
    if not _pynvml_available:
        return []

    result: List[GPUInfo] = []
    try:
        pynvml.nvmlInit()
        gpu_count = pynvml.nvmlDeviceGetCount()

        for i in range(gpu_count):
            handle = pynvml.nvmlDeviceGetHandleByIndex(i)

            gpu_name = pynvml.nvmlDeviceGetName(handle)
            if isinstance(gpu_name, bytes):
                gpu_name = gpu_name.decode("utf-8")

            mem_info = pynvml.nvmlDeviceGetMemoryInfo(handle)
            mem_used_mb = mem_info.used / 1024**2
            mem_total_mb = mem_info.total / 1024**2
            mem_usage = (mem_info.used / mem_info.total) * 100 if mem_info.total > 0 else 0

            util_rates = pynvml.nvmlDeviceGetUtilizationRates(handle)
            gpu_util = util_rates.gpu
            mem_util = util_rates.memory

            result.append(GPUInfo(
                index=i,
                name=gpu_name,
                gpu_util=gpu_util,
                mem_usage=round(mem_usage, 1),
                mem_util=mem_util,
                mem_used_mb=round(mem_used_mb, 1),
                mem_total_mb=round(mem_total_mb, 1),
            ))
    except Exception:
        result = []
    finally:
        try:
            pynvml.nvmlShutdown()
        except Exception:
            pass

    return result



"""系统监控工具 - 获取CPU和内存使用率"""

__version__ = "1.4.2"

"""
changelog:
- 1.4.2: add version to system info
- 1.4.0: add GPU statistic
- 1.3.9: args.interval to background_thread.interval
- 1.3.8: fix use memory_used to calculate statistic
- 1.3.6: fix get_memory_used()
- 1.3.5: add Client.is_alive()
- 1.3.4: 添加ping API, fix
- 1.3.3: 添加ping API
- 1.3.2: client.py增加try-except处理
- 1.3.1: 添加异步客户端
- 1.3.0: 添加客户端
- 1.2.0: 添加系统信息采集线程
- 1.1.0: 添加系统信息采集
- 1.0.0: 初始版本
"""


#! /usr/bin/env python3
# coding: utf-8

from xmovsystemmonitor.client.client import Client
import time
import random


def test_client():
    run_count = 3
    while run_count > 0:
        sleep_seconds = random.randint(3, 5)
        print(f"====test client start [sleep_seconds: {sleep_seconds}] ====")
        client = Client()
        client.start() # 启动监控
        time.sleep(sleep_seconds)
        client.stop() # 停止监控
        print(client.statistic.to_dict())
        print(f"cpu_min: {client.cpu.min}, cpu_max: {client.cpu.max}, cpu_avg: {client.cpu.avg}")
        print(f"memory_min: {client.memory.min}, memory_max: {client.memory.max}, memory_avg: {client.memory.avg}")
        print("====test client end====")
        run_count -= 1




import psutil
from xmovsystemmonitor import __version__

def get_cpu_percent(interval: float = 1.0) -> float:
    """
    获取CPU使用率百分比
    
    Args:
        interval: 采样间隔时间（秒），默认1秒
        
    Returns:
        CPU使用率百分比（0-100）
    """
    return psutil.cpu_percent(interval=interval)


def get_memory_percent() -> float:
    """
    获取内存使用率百分比
    
    Returns:
        内存使用率百分比（0-100）
    """
    return psutil.virtual_memory().percent

def get_memory_used() ->float:
    """
    获取内存使用量（MB）

    Returns:
        内存使用量（MB）
    """
    return psutil.virtual_memory().used / 1024 /1024

def get_system_info() -> dict:
    """
    获取系统信息（CPU和内存使用率）
    
    Returns:
        包含CPU和内存使用率的字典
    """
    from .gpu_helper import get_gpu_info
    return {
        "cpu_percent": get_cpu_percent(),
        "memory_used": get_memory_used(),
        "memory_percent": get_memory_percent(),
        "gpu": [g.to_dict() for g in get_gpu_info()],
        "version": __version__,
    }



# coding: utf-8

import pytest
import asyncio
import aiohttp
from xmovsystemmonitor.client.async_client import AsyncClient

client = AsyncClient()


@pytest.mark.asyncio
async def start():
    await client.start()       


@pytest.mark.asyncio
async def stop():
    await asyncio.sleep(5)
    await client.stop()


@pytest.mark.asyncio
async def test_async_client():
    await client._ensure_session()
    task1 = asyncio.create_task(start())
    task2 = asyncio.create_task(stop())
    result = await asyncio.gather(task1, task2)
    await client._close_session()
    assert client.statistic is not None
    print(client.statistic.to_dict())
    print(f"cpu_min: {client.cpu.min}, cpu_max: {client.cpu.max}, cpu_avg: {client.cpu.avg}")
    print(f"memory_min: {client.memory.min}, memory_max: {client.memory.max}, memory_avg: {client.memory.avg}")


"""命令行入口 - 显示系统监控信息"""

from fastapi import FastAPI
from fastapi.responses import JSONResponse
import uvicorn
import argparse
from xmovsystemmonitor.system_info_monitor_thread import SystemInfoMonitorThread
from xmovsystemmonitor.psutil_helper import get_system_info
from xmovsystemmonitor.schema import Arguments

from xmovsystemmonitor.logger import logger
from xmovsystemmonitor import __version__

def get_args() -> Arguments:
    """获取命令行参数"""
    parser = argparse.ArgumentParser()
    parser.add_argument("--host", type=str, default="0.0.0.0")
    parser.add_argument("--port", type=int, default=8000)
    parser.add_argument("--max-num", type=int, default=3000, help="最大系统信息数量")
    parser.add_argument("--interval", type=float, default=1.0, help="采集间隔时间")
    args = parser.parse_args()
    args = Arguments(
        host=args.host,
        port=args.port,
        max_num=args.max_num,
        interval=args.interval,
    )
    return args

args = get_args()

app = FastAPI()
background_thread = SystemInfoMonitorThread(interval=args.interval, max_system_info_list_length=args.max_num)
background_thread.start()

@app.get("/")
def read_root():
    info = get_system_info()
    return JSONResponse(content=info)

@app.get('/version')
def version():
    return JSONResponse(content={"version": __version__})

@app.get("/start-monitoring")
def start_monitoring():
    """启动监控API"""
    background_thread.reset_system_info_list()
    background_thread.start_to_monitor()
    return JSONResponse(content={"message": "监控已启动"})

@app.get("/stop-monitoring")
def stop_monitoring():
    """停止监控API"""
    background_thread.stop_to_monitor()
    return JSONResponse(content={"message": "监控已停止"})

@app.get("/statistic")
def statistic():
    """获取监控统计信息API（含 CPU、内存、GPU 占用率）"""
    return JSONResponse(content=background_thread.statistic())

@app.get("/ping")
def ping():
    """ping API"""
    return JSONResponse(content={"message": "pong"})


if __name__ == "__main__":
    logger.info(f"xmovsystemmonitor version: {__version__}")
    uvicorn.run(app, host=args.host, port=args.port)



# coding: utf-8

import time
import threading
from xmovsystemmonitor.logger import logger
from xmovsystemmonitor.schema import SystemInfoCollector


class SystemInfoMonitorThread(threading.Thread):
    def __init__(self, interval: float = 1.0, max_system_info_list_length: int = 3000):
        """
        一个用于监测系统CPU和内存使用率的线程，可以用于监控系统性能（内存、CPU）。

        Args:
            interval: 采集间隔时间（秒），默认1秒
            max_system_info_list_length: 最大系统信息列表长度，默认3000
        """
        super().__init__()
        self.running = False
        self.interval = interval
        self.max_system_info_list_length = max_system_info_list_length
        self.system_info_collector = SystemInfoCollector()

    def run(self):
        while True:
            if not self.running:
                time.sleep(self.interval)
                logger.info("监控暂停中...")
                continue
            latest_system_info = self.system_info_collector.collect()
            logger.info(f"监控中, 最新系统信息: {latest_system_info.to_dict()}, 系统信息列表长度: {self.system_info_collector.total_length}")
            time.sleep(self.interval)
    
    def set_interval(self, interval: float):
        """设置采集间隔时间
        
        Args:
            interval: 采集间隔时间（秒）
        """
        self.interval = interval

    def set_max_system_info_list_length(self, max_system_info_list_length: int):
        """设置最大系统信息列表长度
        
        Args:
            max_system_info_list_length: 最大系统信息列表长度
        """
        self.max_system_info_list_length = max_system_info_list_length

    def start_to_monitor(self):
        self.running = True
        logger.info("监控已重置，开始监控")
    
    def reset_system_info_list(self):
        self.system_info_collector.reset_system_info_list()
    
    def stop_to_monitor(self):
        self.running = False
        logger.info("监控已停止")
    
    def statistic(self):
        return self.system_info_collector.statistic()



# coding: utf-8

import requests
from xmovsystemmonitor.schema import Statistic, CPUStatistic, MemoryStatistic


class Client:
    def __init__(self, host: str = "0.0.0.0", port: int = 8000):
        """ 
        一个用于监控系统性能（内存、CPU）的客户端，可以用于监控系统性能（内存、CPU）。

        Args:
            host: 主机地址
            port: 端口号

        Returns:
            Client: 客户端实例
        
        Example:
            client = Client()
            client.start()  # 启动监控
            time.sleep(10)  # 睡眠10秒
            client.stop()  # 停止监控
            print(client.statistic.to_dict())  # 打印统计信息
            print(f"cpu_min: {client.cpu.min}, cpu_max: {client.cpu.max}, cpu_avg: {client.cpu.avg}")
            print(f"memory_min: {client.memory.min}, memory_max: {client.memory.max}, memory_avg: {client.memory.avg}")
        """
        self.host = host
        self.port = port
        self._statistic: Statistic = None

    def _get_system_info(self):
        response = requests.get(f"http://{self.host}:{self.port}/")
        return response.json()

    def _start_monitoring(self):
        try:
            response = requests.get(
                f"http://{self.host}:{self.port}/start-monitoring")
        except Exception as e:
            print(f"start monitoring failed: {e}")
            return None
        return response.json()

    def _stop_monitoring(self):
        try:
            response = requests.get(
                f"http://{self.host}:{self.port}/stop-monitoring")
        except Exception as e:
            print(f"stop monitoring failed: {e}")
            return None
        return response.json()

    def _get_statistic(self):
        try:
            response = requests.get(
                f"http://{self.host}:{self.port}/statistic")
        except Exception as e:
            print(f"get statistic failed: {e}")
            return None
        dat = response.json()
        self._statistic = Statistic.from_dict(dat)
        return dat

    def start(self):
        self._start_monitoring()

    def stop(self):
        self._stop_monitoring()
        self._get_statistic()

    @property
    def statistic(self) -> Statistic:
        return self._statistic

    @property
    def cpu(self) -> CPUStatistic:
        return self._statistic.cpu

    @property
    def memory(self) -> MemoryStatistic:
        return self._statistic.memory

    def is_alive(self):
        try:
            response = requests.get(f"http://{self.host}:{self.port}/ping")
            dat = response.json()
            return dat["message"] == "pong"
        except Exception as e:  
            return False


def main():
    import time
    import random
    client = Client()
    run_count = 3
    while run_count > 0:
        sleep_seconds = random.randint(1, 10)
        print("====start monitoring[sleep_seconds: {sleep_seconds}]====")
        client.start()  # 启动监控
        time.sleep(sleep_seconds)
        client.stop()  # 停止监控
        print(client.statistic.to_dict())
        print(
            f"cpu_min: {client.statistic.cpu.min}, cpu_max: {client.statistic.cpu.max}, cpu_avg: {client.statistic.cpu.avg}")
        print(
            f"memory_min: {client.statistic.memory.min}, memory_max: {client.statistic.memory.max}, memory_avg: {client.statistic.memory.avg}")
        print("====stop monitoring====")
        print('\n\n')
        run_count -= 1


if __name__ == "__main__":
    main()



# coding: utf-8

import asyncio
import aiohttp
from xmovsystemmonitor.schema import Statistic, CPUStatistic, MemoryStatistic

class AsyncClient:
    def __init__(self, host: str = "0.0.0.0", port: int = 8000):
        self.host = host
        self.port = port
        self.session = None
        self._statistic: Statistic = None
    
    async def _ensure_session(self):
        if self.session is None:
            self.session = aiohttp.ClientSession()
    
    async def _close_session(self):
        if self.session is not None:
            await self.session.close()

    async def _get_system_info(self):
        async with self.session.get(f"http://{self.host}:{self.port}/") as response:
            return await response.json()
    
    async def _start_monitoring(self):
        async with self.session.get(f"http://{self.host}:{self.port}/start-monitoring") as response:
            return await response.json()
    
    async def _stop_monitoring(self):
        async with self.session.get(f"http://{self.host}:{self.port}/stop-monitoring") as response:
            return await response.json()    
    
    async def _get_statistic(self):
        async with self.session.get(f"http://{self.host}:{self.port}/statistic") as response:
            dat = await response.json()
            self._statistic = Statistic.from_dict(dat)
            return dat
    
    async def is_alive(self):
        try:
            async with self.session.get(f"http://{self.host}:{self.port}/ping") as response:
                dat = await response.json()
                return dat["message"] == "pong"
        except Exception as e:
            print(f"ping failed: {e}")
            return False

    async def start(self):
        await self._start_monitoring()
    
    async def stop(self):
        await self._stop_monitoring()
        await self._get_statistic()
    
    @property
    def statistic(self) -> Statistic:
        return self._statistic
    
    @property
    def cpu(self) -> CPUStatistic:
        return self._statistic.cpu
    
    @property
    def memory(self) -> MemoryStatistic:
        return self._statistic.memory
    